$ScriptPath = split-path -parent $MyInvocation.MyCommand.Definition

$URI = ""
$DB = "NASHCO"

$OldCSV = "$ScriptPath\old.csv"
$NewCSV = "$ScriptPath\new.csv"
$ConString = "supermag/qqq@$DB"

# Remove HEADO_NOW table

function Group-Compared {
    param([string[]]$Property)

    begin {
        $hashtable = @{}
    }

    process {
        $key = $(foreach($prop in $Property) { $_.$prop }) -join ','
        If (!($hashtable.ContainsKey($key))) {
            $hashtable[$key] = @{
                extId = $_.extId;
                article = $_.article;
                amount = "0"
            }
        }
        if ($_.SideIndicator = "=>") {
            $hashtable[$key].amount = $_.amount
        }
    }

    end {
        $hashtable.Values
    }
}

$prev_nls_lang = $env:nls_lang
$env:nls_lang = '.AL32UTF8'
( sqlplus -silent $ConString "@$ScriptPath\price.sql" "$ScriptPath\Price\\" )# | Out-Null
$env:nls_lang = $prev_nls_lang

$First = if (Test-Path $OldCSV]) { $False } else { $True }

( sqlplus -silent $ConString "@$ScriptPath\remains.sql" "$ScriptPath\new.csv" $First )# | Out-Null
$NewRemains = Import-Csv $NewCSV ';' -Header extId, article, amount
if (!$First) {
    $OldRemains = Import-Csv $OldCSV ';' -Header extId, article, amount
    
    function Value-On-SideIndicator($Object, $SideIndicator) {
        $Group = $Object.group | ? { $_.SideIndicator -eq $SideIndicator }
        if ($Group -ne $null) { $Group.amount } else { "0" }
    }
    
    Get-Date
    $Diff1 = Compare-Object $OldRemains $NewRemains -Property extId, article, amount 
    Get-Date
    $Diff = $Diff1 | Group-Compared -Property extId, article
    Get-Date
} else {
    $Diff = $NewRemains
}

$BodyTmplt = @"
{{
    "id": "{0}",
    "method": "inventoryUpdateBatch",
    "params": {{
        "items": [
{1}
        ]
    }},
    "jsonrpc": "2.0"
}}
"@
$ItemTmplt = @"
            {{
                "store_ext_id": "{0}",
                "price_ext_id": "{1}",
                "snapshot_datetime": "{2}",
                "qty": "{3}"
            }}
"@
function Out-Log
{
    Param ([string]$LogString)
    get-ChildItem $ScriptPath | where {$_.name -like "remains\.\d{8}\.log" -and $_.CreationTime -lt (get-date).adddays(-7)} | Remove-Item -Force
    $LogFile = "$ScriptPath\remains.{0}.log" -f (Get-Date).toString("yyyyMMdd")
    $Stamp = (Get-Date).toString("yyyy/MM/dd HH:mm:ss")
    $LogMessage = "$Stamp $LogString"
    Add-content $LogFile -Value $LogMessage
}
function MaxOf($a, $b) { if ($a -lt $b) { $a } else { $b } }
For ($i=0; $i -lt $Diff.Length; $i+=1000) {
    $Items = ($Diff[$i..$(MaxOf ($i+999) $Diff.Length)] | % {
        $ItemTmplt -f $_.extId, $_.article, (Get-Date -Format 'o'), $_.amount
    }) -join ",`r`n"
	$Message = ($BodyTmplt -f $(-join (0..9 | Get-Random -Count 16)), $Items) -replace '\s'
    Out-Log "$Message"
    $Body = [byte[]][char[]]$Message
    $WebRequest = [System.Net.WebRequest]::Create($URI)
    $WebRequest.Method = "POST"
    $WebRequest.ContentType = "application/json"
    $Upload = $WebRequest.GetRequestStream()
    $Upload.Write($Body, 0, $Body.Length)
    $Upload.Flush()
    $Upload.Close()
    $Response = $WebRequest.GetResponse()
    $StreamReader = [System.IO.StreamReader]($Response.GetResponseStream())
    $Data = $StreamReader.ReadToEnd()
    $StreamReader.Close()
    $Response.Close()
    Out-Log $([regex]::Unescape($Data))
	#Write-Host ([regex]::Unescape($Data) | Format-Table | Out-String)
}

if (Test-Path $OldCSV) { Remove-Item $OldCSV -Force }
Rename-Item $NewCSV $OldCSV -Force